@@ -1,4 +1,4 @@
-/* This file is part of the KDE project
+/*
    Copyright (C) 2000 David Faure <faure@kde.org>
 
    This library is free software; you can redistribute it and/or
@@ -21,7 +21,11 @@
 #define __slavebase_h
 
 #include <kurl.h>
-#include "kio/global.h"
+#include <kconfigbase.h>
+#include <kio/global.h>
+#include <kio/authinfo.h>
+
+class DCOPClient;
 
 namespace KIO {
 
@@ -43,6 +47,12 @@ public:
     SlaveBase( const QCString &protocol, const QCString &pool_socket, const QCString &app_socket);
     virtual ~SlaveBase();
 
+    /**
+     * @internal
+     * Terminate the slave by calling the destructor and then ::exit()
+     */
+    void exit();
+
     /**
      * @internal
      */
@@ -63,14 +73,17 @@ public:
     ///////////
 
     /**
-     * Call to send data in the slave (i.e. in get).
+     * Sends data in the slave to the job (i.e. in get).
+     *
+     * To signal end of data, simply send an empty
+     * QByteArray().
+     *
      * @param data the data read by the slave
-     * Send an empty QByteArray() to signal end of data.
      */
     void data( const QByteArray &data );
 
     /**
-     * Call to ask for data (in put)
+     * Asks for data from the job.
      * @see readData
      */
     void dataReq( );
@@ -78,12 +91,24 @@ public:
     /**
      * Call to signal an error.
      * This also finishes the job, no need to call finished.
+     *
+     * If the Error code is KIO::ERR_SLAVE_DEFINED then the
+     * _text should contain the complete translated text of
+     * of the error message.  This message will be displayed
+     * in an KTextBrowser which allows rich text complete
+     * with hyper links.  Email links will call the default
+     * mailer, "exec:/command arg1 arg2" will be forked and
+     * all other links will call the default browser.
+     *
+     * @see KIO::Error
+     * @see KTextBrowser
+     * @param _errid the error code from KIO::Error
+     * @param _text the rich text error message
      */
     void error( int _errid, const QString &_text );
 
     /**
      * Call in openConnection, if you reimplement it, when you're done.
-     * Currently unused.
      */
     void connected();
 
@@ -107,7 +132,10 @@ public:
     void slaveStatus(const QString &host, bool connected);
 
     /**
-     * Call this once in stat()
+     * Call this from stat() to express details about an object, the
+     * UDSEntry customarily contains the atoms describing file name, size,
+     * mimetype, etc.
+     * @param _entry The UDSEntry containing all of the object attributes.
      */
     void statEntry( const UDSEntry& _entry );
 
@@ -124,7 +152,7 @@ public:
      * In this case, the boolean returns whether we can indeed resume or not
      * (we can't if the protocol doing the get() doesn't support setting an offset)
      */
-    bool canResume( unsigned long offset );
+    bool canResume( KIO::filesize_t offset );
 
     /*
      * Call this at the beginning of get(), if the "resume" metadata was set
@@ -137,20 +165,35 @@ public:
     ///////////
 
     /**
-     * Call this in @ref get and @ref copy, to give the total size
+     * Call this in get and copy, to give the total size
      * of the file
-     * Call in @ref listDir too, when you know the total number of items.
+     * Call in listDir too, when you know the total number of items.
      */
-    void totalSize( unsigned long _bytes );
+    void totalSize( KIO::filesize_t _bytes );
     /**
-     * Call this during @ref get and @ref copy, once in a while,
+     * Call this during get and copy, once in a while,
      * to give some info about the current state.
-     * Don't emit it in @ref listDir, @ref listEntries speaks for itself.
+     * Don't emit it in listDir, listEntries speaks for itself.
      */
-    void processedSize( unsigned long _bytes );
+    void processedSize( KIO::filesize_t _bytes );
+
     /**
-     * Call this in @ref get and @ref copy, to give the current transfer
-     * speed. Usually worked out as processed_size / ( t - t_start )
+     * Only use this if you can't know in advance the size of the
+     * copied data. For example, if you're doing variable bitrate
+     * compression of the source.
+     *
+     * STUB ! Currently unimplemented. Here now for binary compatibility.
+     *
+     * Call this during get and copy, once in a while,
+     * to give some info about the current state.
+     * Don't emit it in listDir, listEntries speaks for itself.
+     */
+    void processedPercent( float percent );
+
+    /**
+     * Call this in get and copy, to give the current transfer
+     * speed, but only if it can't be calculated out of the size you
+     * passed to processedSize (in most cases you don't want to call it)
      */
     void speed( unsigned long _bytes_per_second );
 
@@ -168,13 +211,11 @@ public:
     void errorPage();
 
     /**
-     * Call this in @ref mimetype, when you know the mimetype.
-     * See @ref mimetype about other ways to implement it.
+     * Call this in mimetype, when you know the mimetype.
+     * See mimetype about other ways to implement it.
      */
     void mimeType( const QString &_type );
 
-    void gettingFile( const QString &_file ); // probably obsolete ?
-
     /**
      * Call to signal a warning, to be displayed in a dialog box.
      */
@@ -186,7 +227,7 @@ public:
      */
     void infoMessage( const QString &msg );
 
-    enum { QuestionYesNo = 1, WarningYesNo = 2, WarningContinueCancel = 3, WarningYesNoCancel = 4, Information = 5, SSLMessageBox = 6 };
+    enum MessageBoxType { QuestionYesNo = 1, WarningYesNo = 2, WarningContinueCancel = 3, WarningYesNoCancel = 4, Information = 5, SSLMessageBox = 6 };
 
     /**
      * Call this to show a message box from the slave (it will in fact be handled
@@ -203,7 +244,7 @@ public:
      *       and for Information, none is used.
      * @return a button code, as defined in KMessageBox, or 0 on communication error.
      */
-    int messageBox( int type, const QString &text,
+    int messageBox( MessageBoxType type, const QString &text,
                     const QString &caption = QString::null,
                     const QString &buttonYes = QString::null,
                     const QString &buttonNo = QString::null );
@@ -215,12 +256,43 @@ public:
     void setMetaData(const QString &key, const QString &value);
 
     /**
-     * Queries for meta-data send by the application to the slave.
+     * Queries for the existence of a certain config/meta-data entry
+     * send by the application to the slave.
+     * @since 3.2
+     */
+    bool hasMetaData(const QString &key) const;
+
+    /**
+     * Queries for config/meta-data send by the application to the slave.
+     * @since 3.2
+     */
+    QString metaData(const QString &key) const;
+
+    /**
+     * @obsolete kept for binary compatibility
+     * Queries for the existence of a certain config/meta-data entry
+     * send by the application to the slave.
+     */
+    bool hasMetaData(const QString &key);
+
+    /**
+     * @obsolete kept for binary compatibility
+     * Queries for config/meta-data send by the application to the slave.
      */
     QString metaData(const QString &key);
 
+    /**
+     * Returns a configuration object to query config/meta-data information
+     * from.
+     *
+     * The application provides the slave with all configuration information
+     * relevant for the current protocol and host.
+     */
+    KConfigBase* config();
+
     ///////////
-    // Commands sent by the job, the slave has to override what it wants to implement
+    // Commands sent by the job, the slave has to
+    // override what it wants to implement
     ///////////
 
     /**
@@ -243,13 +315,22 @@ public:
 
     /**
      * Opens the connection (forced)
-     * Currently unused.
+     * When this function gets called the slave is operating in
+     * connection-oriented mode.
+     * When a connection gets lost while the slave operates in
+     * connection oriented mode, the slave should report
+     * ERR_CONNECTION_BROKEN instead of reconnecting. The user is
+     * expected to disconnect the slave in the error handler.
      */
     virtual void openConnection();
 
     /**
      * Closes the connection (forced)
-     * Currently unused.
+     * Called when the application disconnects the slave to close
+     * any open network connections.
+     *
+     * When the slave was operating in connection-oriented mode,
+     * it should reset itself to connectionless (default) mode.
      */
     virtual void closeConnection();
 
@@ -257,22 +338,24 @@ public:
      * get, aka read.
      * @param url the full url for this request. Host, port and user of the URL
      *        can be assumed to be the same as in the last setHost() call.
-     * The slave emits the data through @ref data
+     * The slave emits the data through data
      */
     virtual void get( const KURL& url );
 
     /**
      * put, aka write.
-     * @param path where to write the file (decoded)
+     * @param url where to write the file (decoded)
      * @param permissions may be -1. In this case no special permission mode is set.
-     * @param overwrite if true, any existing file will be overwritten
+     * @param overwrite if true, any existing file will be overwritten.
+     * If the file indeed already exists, the slave should NOT apply the
+     * permissions change to it.
      * @param resume
      */
     virtual void put( const KURL& url, int permissions, bool overwrite, bool resume );
 
     /**
      * Finds all details for one file or directory.
-     * The information returned is the same as what @ref listDir returns,
+     * The information returned is the same as what listDir returns,
      * but only for one file or directory.
      */
     virtual void stat( const KURL& url );
@@ -284,7 +367,7 @@ public:
      * should send a block of data big enough to be able
      * to determine the mimetype.
      *
-     * If the slave doesn't reimplement it, a @ref get will
+     * If the slave doesn't reimplement it, a get will
      * be issued, i.e. the whole file will be downloaded before
      * determining the mimetype on it - this is obviously not a
      * good thing in most cases.
@@ -292,17 +375,17 @@ public:
     virtual void mimetype( const KURL& url );
 
     /**
-     * Lists the contents of @p path.
+     * Lists the contents of @p url.
      * The slave should emit ERR_CANNOT_ENTER_DIRECTORY if it doesn't exist,
      * if we don't have enough permissions, or if it is a file
-     * It should also emit @ref totalFiles as soon as it knows how many
+     * It should also emit totalFiles as soon as it knows how many
      * files it will list.
      */
     virtual void listDir( const KURL& url );
 
     /**
      * Create a directory
-     * @param path path to the directory to create
+     * @param url path to the directory to create
      * @param permissions the permissions to set after creating the directory
      * (-1 if no permissions to be set)
      * The slave emits ERR_COULD_NOT_MKDIR if failure.
@@ -348,7 +431,7 @@ public:
 
     /**
      * Delete a file or directory.
-     * @param path file/directory to delete
+     * @param url file/directory to delete
      * @param isfile if true, a file should be deleted.
      *               if false, a directory should be deleted.
      */
@@ -362,7 +445,16 @@ public:
      *        slave, but usually starts with an int for the command number.
      * Document your slave's commands, at least in its header file.
      */
-    virtual void special( const QByteArray & );
+    virtual void special( const QByteArray & data );
+
+    /**
+     * Used for multiple get. Currently only used foir HTTP pielining
+     * support.
+     *
+     * @param data packed data; Contains number of URLs to fetch, and for
+     * each URL the URL itself and its associated MetaData.
+     */
+    virtual void multiGet( const QByteArray & data );
 
     /**
      * Called to get the status of the slave. Slave should respond
@@ -376,6 +468,45 @@ public:
      */
     virtual void reparseConfiguration();
 
+
+    /**
+     * @return timeout value for connecting to remote host.
+     */
+    int connectTimeout();
+
+    /**
+     * @return timeout value for connecting to proxy in secs.
+     */
+    int proxyConnectTimeout();
+
+    /**
+     * @return timeout value for read from first data from
+     * remote host in seconds.
+     */
+    int responseTimeout();
+
+    /**
+     * @return timeout value for read from subsequent data from
+     * remote host in secs.
+     */
+    int readTimeout();
+
+    /**
+     * This function sets a timeout of @p timeout seconds and calls
+     * special(data) when the timeout occurs as if it was called by the
+     * application.
+     *
+     * A timeout can only occur when the slave is waiting for a command
+     * from the application.
+     *
+     * Specifying a negative timeout cancels a pending timeout.
+     *
+     * Only one timeout at a time is supported, setting a timeout
+     * cancels any pending timeout.
+     * @since 3.1
+     */
+    void setTimeoutSpecialCommand(int timeout, const QByteArray &data=QByteArray());
+
     /**
      * @internal
      */
@@ -399,7 +530,7 @@ public:
     virtual void dispatch( int command, const QByteArray &data );
 
     /**
-     * Read data send by the job, after a @ref dataReq
+     * Read data send by the job, after a dataReq
      *
      * @param buffer buffer where data is stored
      * @return 0 on end of data,
@@ -408,7 +539,6 @@ public:
      **/
     int readData( QByteArray &buffer );
 
-protected:
     /**
      * internal function to be called by the slave.
      * It collects entries and emits them via listEntries
@@ -429,133 +559,145 @@ protected:
     void disconnectSlave();
 
     /**
-     * Checks whether the password daemon kdesud is
-     * up and running or can be started if it is not.
+     * Prompt the user for Authorization info (login & password).
      *
-     * @return true if password daemon is/can be started successfully.
+     * Use this function to request authorization information from
+     * the end user. You can also pass an error message which explains
+     * why a previous authorization attempt failed. Here is a very
+     * simple example:
+     *
+     * \code
+     * KIO::AuthInfo authInfo;
+     * if ( openPassDlg( authInfo ) )
+     * {
+     *    kdDebug() << QString::fromLatin1("User: ")
+     *              << authInfo.username << endl;
+     *    kdDebug() << QString::fromLatin1("Password: ")
+     *              << QString::fromLatin1("Not displayed here!") << endl;
+     * }
+     * \endcode
+     *
+     * You can also preset some values like the username, caption or
+     * comment as follows:
+     *
+     * \code
+     * KIO::AuthInfo authInfo;
+     * authInfo.caption= "Acme Password Dialog";
+     * authInfo.username= "Wile E. Coyote";
+     * QString errorMsg = "You entered an incorrect password.";
+     * if ( openPassDlg( authInfo, errorMsg ) )
+     * {
+     *    kdDebug() << QString::fromLatin1("User: ")
+     *              << authInfo.username << endl;
+     *    kdDebug() << QString::fromLatin1("Password: ")
+     *              << QString::fromLatin1("Not displayed here!") << endl;
+     * }
+     * \endcode
+     *
+     * NOTE: A call to this function can fail and return @p false,
+     * if the UIServer could not be started for whatever reason.
+     *
+     * @param info  See AuthInfo.
+     * @param errorMsg Error message to show
+     * @return      @p true if user clicks on "OK", @p false otherwsie.
+     * @since 3.1
+     */
+    bool openPassDlg( KIO::AuthInfo& info, const QString &errorMsg );
+
+    /**
+     * Same as above function except it does not need error message.
+     * BIC: Combine this function with the above for KDE4.
+     */
+    bool openPassDlg( KIO::AuthInfo& info );
+
+    /**
+     * Checks for cached authentication based on parameters
+     * given by @p info.
+     *
+     * Use this function to check if any cached password exists
+     * for the URL given by @p info.  If @p AuthInfo::realmValue
+     * and/or @p AuthInfo::verifyPath flag is specified, then
+     * they will also be factored in determining the presence
+     * of a cached password.  Note that @p Auth::url is a required
+     * parameter when attempting to check for cached authorization
+     * info. Here is a simple example:
+     *
+     * \code
+     * AuthInfo info;
+     * info.url = KURL("http://www.foobar.org/foo/bar");
+     * info.username = "somename";
+     * info.verifyPath = true;
+     * if ( !checkCachedAuthentication( info ) )
+     * {
+     *    if ( !openPassDlg(info) )
+     *     ....
+     * }
+     * \endcode
+     *
+     * @param       info See AuthInfo.
+     * @return      @p true if cached Authorization is found, false otherwise.
+     */
+    bool checkCachedAuthentication( AuthInfo& info );
+
+    /**
+     * Explicitly store authentication information. openPassDlg already
+     * stores password information automatically, you only need to call
+     * this function if you want to store authentication information that
+     * is different from the information returned by openPassDlg.
+     */
+    bool cacheAuthentication( const AuthInfo& info );
+
+    /**
+     * @obsolete as of 3.1.
+     * TODO: Remove before KDE 4.0
      */
     bool pingCacheDaemon() const;
 
     /**
-     * Prompts the user for Authentication info (login & password).
-     *
-     * This function attempts to prompt the user for a password
-     * and returns true if the user complies (clicks OK) or
-     * false otherwise (clicks Cancel).
-     *
-     * Exception: A call to this function can also fail, result
-     * in @p false, if the UIServer could not be started for some
-     * reason.
-     *
-     * @param msg     i18n'ed message to explain the dialog box
-     * @param user    user name, in and out
-     * @param pass    password, in and out
-     * @param lockUserName flag used to enable/disable the username field.
-     *
-     * @return        @p true on if successful, @p false otherwise
-     */
-    bool openPassDlg( const QString& msg, QString& user, QString& passwd, bool lockUserName = false );
-
-    /**
-     * Checks for any cached Authentication.
-     *
-     * @param url           url for which to check cached Authentication.
-     * @param user          cached user name.
-     * @param passw         cached password.
-     * @param realm         unique key to distinguish protection spaces (ex: HTTP Realm values)
-     * @param extra         extra info to store (ex: Authentication strings in Digest Authentication )
-     * @param verify_path   if true, check new url contains cached url (== same protection space)
-     *
-     * @return              @p true if a cached Authentication is found
-     */
-    bool checkCachedAuthentication( const KURL& url,
-                                    QString& user,
-                                    QString& passwd,
-                                    QString& realm,
-                                    QString& extra,
-                                    bool verify_path = true );
-
-    /**
-     * Same as above except in the number of arguments it takes.
-     *
-     * This is a convenience method for protocols that have simple
-     * Authentication and do not require complex caching schemes
-     * such as ftp
-     *
-     * @param url           url for which to check cached Authentication.
-     * @param user          cached user name.
-     * @param passw         cached password.
-     *
-     * @return              @p true if cached Authentication if found
-     */
-    bool checkCachedAuthentication( const KURL& url,
-                                    QString& user,
-                                    QString& passwd);
-
-    /**
-     * Caches Authentication information in kdesu daemon.
-     *
-     * Authentication caching is based on the following criteria:
-     *
-     *   i.)  The protocol as part of the key generation.  This will
-     *        reduce the chances of inadvertantly sending password to the
-     *        incorrect server. Thus, http://www.foobar.org and
-     *        ftp://www.foobar.org are treated as different request sites
-     *        even if the same Authentication is assigned to the user for
-     *        accessing both locations.
-     *
-     *  ii.)  Cache different servers on the the same host but with different
-     *        port numbers. For example, one might have multiple web-based admin
-     *        tools, such as Webmin and SWAT, on the same server with different
-     *        port numbers.
-     *
-     * iii.)  Use reference counting to keep track of all applications that are
-     *        requesting password caching for the same location instead of storing
-     *        duplicate enteries.  The cached password can then be deleted when the
-     *        last application referencing it is destroyed.
-     *
-     *  iv.)  Allow redundant password caching for the same host based on heirarchy
-     *        such that protocols, such as HTTP, can store Authentication info
-     *	       for multiple password protected content within the same site.  For example,
-     *        http://foo.org/foo/foo.html and http://foo.org/foo/bar/bar.html would fall
-     *        under the same protection space while http://foo.org/foobar/foo.html would
-     *        not and hence gets a different entry. Refer to RFC 2617 for further details.
-     *
-     * @param url       url for which Authentication is to be cached.
-     * @param user      user name to be cached.
-     * @param passw     password to be cached.
-     * @param realm     unique key to distinguish protection spaces (ex: HTTP Realm values)
-     * @param extra     extra info to store (ex: Authentication strings in Digest Authentication )
-     *
-     * @return          @p true if Authentication was sucessfully cached
-     */
-    bool cacheAuthentication( const KURL& url,
-                              const QString& user,
-                              const QString& passwd,
-                              const QString& realm = QString::null,
-                              const QString& extra = QString::null );
-
-    /**
+     * @obsolete as of 3.1. Use openPassDlg instead.
+     * TODO: Remove before KDE 4.0
      * Creates a basic key to be used to cache the password.
-     *
-     * @param url   URL for which a caching key should be generated.
-     * @return      NULL if @p url is malformed, otherwise the generated key.
+     * @param url   the url from which the key is supposed to be generated
      */
     QString createAuthCacheKey( const KURL& url );
 
     /**
-     * Deletes any cached keys for the given group.
+     * @obsolete as of 3.1. Use openPassDlg instead.
+     * TODO: Remove before KDE 4.0
      *
-     * @param grpname   group name for which cached Authentication is to be deleted.
+     * Cache authentication information is now stored automatically
+     * by openPassDlg.
      */
-    void delCachedAuthentication( const QString& grpname );
+    void sendAuthenticationKey( const QCString& gKey, const QCString& key, bool keep );
+
+    /**
+     * @obsolete as of 3.1. Use openPassDlg instead.
+     * TODO: Remove before KDE 4.0
+     *
+     * Cached authentication information is now session based and
+     * removed automatically when a given session ends, i.e. the
+     * application is closed.
+     */
+    void delCachedAuthentication( const QString& key );
+
+    /**
+     * @obsolete as of 3.1. Use openPassDlg instead.
+     * TODO: Remove before KDE 4.0
+     */
+    void setMultipleAuthCaching( bool ) {};
+
+    /**
+     * @obsolete as of 3.1. Use openPassDlg instead.
+     * TODO: Remove before KDE 4.0
+     */
+    bool multipleAuthCaching() const { return false; }
 
     /**
      * Used by the slave to check if it can connect
      * to a given host. This should be called where the slave is ready
-     * to do a ::connect() on a socket. For each call to @ref
+     * to do a ::connect() on a socket. For each call to
      * requestNetwork must exist a matching call to
-     * @ref dropNetwork, or the system will stay online until
+     * dropNetwork, or the system will stay online until
      * KNetMgr gets closed (or the SlaveBase gets destructed)!
      *
      * If KNetMgr is not running, then this is a no-op and returns true
@@ -572,8 +714,8 @@ protected:
 
     /**
      * Used by the slave to withdraw a connection requested by
-     * @ref requestNetwork. This function cancels the last call to
-     * @ref requestNetwork. If a client uses more than one internet
+     * requestNetwork. This function cancels the last call to
+     * requestNetwork. If a client uses more than one internet
      * connection, it must use dropNetwork(host) to
      * stop each request.
      *
@@ -585,13 +727,23 @@ protected:
      * */
     void dropNetwork(const QString& host = QString::null);
 
+    /**
+     * Return the dcop client used by this slave.
+     * @since 3.1
+     */
+    DCOPClient *dcopClient();
+
     /**
      * Wait for an answer to our request, until we get @p expected1 or @p expected2
      * @return the result from readData, as well as the cmd in *pCmd if set, and the data in @p data
      */
     int waitForAnswer( int expected1, int expected2, QByteArray & data, int * pCmd = 0 );
 
-protected:
+    /**
+     * Internal function to transmit meta data to the application.
+     */
+    void sendMetaData();
+
     /**
      * Name of the protocol supported by this slave
      */
@@ -599,24 +751,38 @@ protected:
 
     Connection * m_pConnection;
 
-    /**
-     * Internal function to transmit meta data to the application.
-     */
-    void sendMetaData();
+    MetaData mOutgoingMetaData;
+    MetaData mIncomingMetaData;
 
-private:
+    /** If your ioslave was killed by a signal, wasKilled() returns true.
+     Check it regularly in lengthy functions (e.g. in get();) and return
+     as fast as possible from this function if wasKilled() returns true.
+     This will ensure that your slave destructor will be called correctly.
+     @since 3.1
+     */
+    bool wasKilled() const;
+
+    /** Internally used.
+     * @internal
+     * @since 3.1
+     */
+    void setKillFlag();
+
+protected:
     UDSEntryList pendingListEntries;
     uint listEntryCurrentSize;
     long listEntry_sec, listEntry_usec;
     Connection *appconn;
     QString mPoolSocket;
     QString mAppSocket;
-    MetaData mOutgoingMetaData;
-    MetaData mIncomingMetaData;
     bool mConnectedToApp;
+    static long s_seqNr;
+    virtual void virtual_hook( int id, void* data );
+
+private:
     SlaveBasePrivate *d;
 };
 
-};
+}
 
 #endif
